#!/usr/bin/env python3
# -*- coding: utf-8 -*-

name = 'channel2pdf'

import os
from .find_resource import findResource
from .article import getArticleHtml, getCustomHtml
from .index import getIndexHtml
from datetime import date
from telegram_util import cleanFileName

if os.name == 'posix':
	ebook_convert_app = '/Applications/calibre.app/Contents/MacOS/ebook-convert'
else:
	ebook_convert_app = 'ebook-convert'

def gen(source, ebook_convert_app=ebook_convert_app, additional_setting='', filename_suffix = ''):
	name, links, pics, texts = findResource(source)
	filename = '%s_%s' % (date.today().strftime("%m%d"), name) + filename_suffix
	filename = filename.replace(' ', '_')

	os.system('rm -rf html_result')	
	os.system('mkdir html_result > /dev/null 2>&1')

	links['图集精选'] = '图集精选'
	links['消息精选'] = '消息精选'
	for link, title in links.copy().items():
		if title == '图集精选':
			html = getCustomHtml(title, pics, filename + '.html')
		elif title == '消息精选':
			html = getCustomHtml(title, texts, filename + '.html')
		else:
			html = getArticleHtml(title, link, filename + '.html')
		if html:
			with open('html_result/%s.html' % cleanFileName(title), 'w') as f:
				f.write(html)
		else:
			del links[link]

	index_html_name = 'html_result/%s.html' % filename
	with open(index_html_name, 'w') as f:
		f.write(getIndexHtml(name, source, links))

	os.system('mkdir pdf_result > /dev/null 2>&1')
	pdf_name = 'pdf_result/%s.pdf' % filename
	os.system('%s %s %s %s > /dev/null 2>&1' % \
		(ebook_convert_app, index_html_name, pdf_name, additional_setting))
	return pdf_name
		




from bs4 import BeautifulSoup
import readee

def getHtml(name, soup, index_loc):
	return '''
<html>
	<body>
		<title>%s</title>
		<h1>%s</h1>
		<div><a href="%s">返回目录</a></div>
		%s
		<div><br/><a href="%s">返回目录</a></div>
	</body>
</html>
	''' % (name, name, index_loc, str(soup), index_loc)

def getCustomHtml(name, content, index_loc):
	if not content:
		return
	raw = []
	for x, y in content:
		raw.append('<div>%s%s</div>' % (x, y))
	return getHtml(name, '<br/><br/>' + '<br/><hr/><br/>'.join(raw), index_loc)

def getArticleHtml(name, link_list, index_loc):
	result = ''
	for link in link_list: 
		soup = readee.export(link)
		if len(soup.text) < 100:
			continue
		result += str(soup)
	if not result:
		return
	return getHtml(name, '''
		%s
		<div><br/><a href="%s">原文</a></div>
	''' % (result, link), index_loc)


import os
from bs4 import BeautifulSoup
import yaml
from telegram_util import matchKey
import cached_url
from .find_links import findLinks
from datetime import date
import re

LINK_PREFIX = 'https://t.me/s/'

def findSrc(raw):
	pivot1 = "background-image:url('"
	index = raw.find(pivot1)
	raw = raw[index + len(pivot1):]
	pivot2 = "')"
	index = raw.find(pivot2)
	return raw[:index]

def getLink(post):
	print('t.me/' + post.getKey(), post.text, post.link)
	if not post.link:
		return
	return tuple([item['href'] for item in 
		soup.find_all('a') if item.get('href')]), post.link

def findResource(source):
	soup = BeautifulSoup(cached_url.get(LINK_PREFIX + source), 'html.parser')
	name = soup.find('meta', {'property': 'og:title'})['content']
	links = {}
	for item in soup.find_all('a', class_='tgme_widget_message_link_preview'):
		if 'telegra.ph' not in item['href']:
			continue
		title = item.find('div', class_='link_preview_title').text
		links[(item['href'],)] = title
	pics = []
	for item in soup.find_all('div', class_='tgme_widget_message_bubble'):
		imgs = []
		for pic in item.find_all('a', class_='tgme_widget_message_photo_wrap'):
			imgs.append('<figure><img src="%s"/></figure>' % findSrc(pic['style']))
		text = item.find('div', class_='tgme_widget_message_text')
		if imgs:
			pics.append((''.join(imgs), text or ''))
	texts = []
	for item in soup.find_all('div', class_='tgme_widget_message_wrap'):
		if item.find('a', class_='tgme_widget_message_photo_wrap'):
			continue
		preview = item.find('a', class_='tgme_widget_message_link_preview')
		if preview:
			preview.name = 'div'
		text = item.find('div', class_='tgme_widget_message_text')
		texts.append((text, preview or ''))
	if len(links) == 0:
		links = findLinks(source)
	if name == 'MengyShare':
		name = '端传媒'
	return name, links, pics, texts

	



from datetime import date
from bs4 import BeautifulSoup
import re
from telegram_util import cleanFileName

def isCN(title):
	if re.search(u'[\u4e00-\u9fff]', title):
		return True
	return False

def getIndexHtml(name, source, links):
	today = date.today().strftime("%m%d")

	index_html = '''
<html>
   <body>
	 <h1>%s %s</h1>
	 <br/>
	 <p style="text-indent:0pt">
	 </p>
	 <br/>
	 <br/>
	 <p style="text-indent:0pt">
		来源：
		<a href="https://t.me/%s">%s</a>
	 </p>
	 <p style="text-indent:0pt">
		<a href="https://github.com/gaoyunzhi/telegram_news_2_pdf_bot/tree/master/pdf_result">镜像</a>
	 </p>
   </body>
</html>
	''' % (name, today, source, name)

	soup = BeautifulSoup(index_html, 'html.parser')
	content_list = soup.find('p')
	titles = []
	for _, title in links.items():
		if '精选' in title:
			titles.append((1, title))
		elif isCN(title):
			titles.append((0, title))
		else:
			titles.append((2, title))
	for _, title in sorted(titles):
		item = '<a href="%s.html">%s</a>' % \
			(cleanFileName(title), cleanFileName(title))
		content_list.append(BeautifulSoup(item, 'html.parser'))
		content_list.append(BeautifulSoup('<br/><br/>', 'html.parser'))
	return str(soup)


from telethon import TelegramClient
from telethon.tl.functions.messages import GetHistoryRequest
import yaml
import asyncio

with open('credential') as f:
    credential = yaml.load(f, Loader=yaml.FullLoader)

loop = asyncio.new_event_loop()
asyncio.set_event_loop(loop)
client = TelegramClient('session_file', credential['api_id'], credential['api_hash'])
client.start(password=credential['password'])

async def findLinksAsync(source):
    channel_entity=await client.get_entity(source)
    posts = await client(GetHistoryRequest(peer=channel_entity, limit=30, # change to 30
        offset_date=None, offset_id=0, max_id=0, min_id=0, add_offset=0, hash=0))
    return posts

def findLinks(source):
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    r = loop.run_until_complete(findLinksAsync(source))
    links = {}
    for message in r.messages:
        try:
            message.media.webpage
        except:
            continue
        links[tuple([item.url for item in message.entities])] = (
            message.media.webpage.title[11:-21]).strip()
    return links
    